@@ -7,6 +7,7 @@
 #include "connection.h"
 #include "init.h"
 #include "io.h"
+#include "log.h"
 
 extern Connection *conn;
 extern int conn_size, maxfd;
@@ -14,10 +15,12 @@ extern int max_simul_con;
 extern int timeout;
 extern int lfd;
 extern int sp_base;
+extern int do_log;
 extern fd_set rdset, wrset;
 extern int sndbufsz;
+extern char *AuthData;
+extern char *LogFile;
 extern R_RSA_PRIVATE_KEY ServerPrivateKey;
 extern int ipv6support;
 
 #endif
-
