@@ -6,42 +6,28 @@
 void Connection_alloc(void) {
    
 #ifdef DEBUG
-   if(debug)
-      err_msg("Connection_alloc: allocating new structure, conn_size=%d", conn_size);
+   err_msg("Connection_alloc: allocating new structure, conn_size=%d", conn_size);
 #endif
 
-   if(conn == NULL)
-      conn = calloc(1, sizeof(Connection));
-   else
-      conn = realloc(conn, (conn_size + 1) * sizeof(Connection));
-
-   if(conn == NULL)
-      err_sys("Connection_alloc: Memory allocation error");
-
-   conn[conn_size].rbuf = malloc(sndbufsz);
-   conn[conn_size].lbuf = malloc(sndbufsz);
-
-   if((conn[conn_size].lbuf == NULL) || (conn[conn_size].rbuf == NULL))
-      err_sys("Connection_alloc: Memory allocation error");
+   conn = Realloc(conn, (conn_size + 1) * sizeof(Connection));
 
+   memset(&conn[conn_size], 0, sizeof(Connection));
    conn[conn_size].status = ST_NOTUSED;
    conn_size++;
 }
 
 
-/* This function searches for a free Connection structure, or allocates a new one
-   It also sets the time the socket is added, for the timeout functionality
-   Then it eventually updates maxfd. It adds the socketdescriptor 
-   to the readset and the index is returned.
+/* This function searches for a free Connection structure, or allocates a new one.
+   It also sets the time the socket is added, for the timeout functionality.
+   It sets the socking to nonblocking mode. Then it eventually updates maxfd.
+   It adds the socketdescriptor to the readset and the index is returned.
  */
 int Connection_add(int localfd) {
    int i;
 
 #ifdef DEBUG
-   if(debug)
-      err_msg("Connection_add: Adding Connection structure for fd=%d", localfd);
+   err_msg("Connection_add: Adding Connection structure for fd=%d", localfd);
 #endif
-
    if(conn == NULL)
       Connection_alloc();
 
@@ -52,6 +38,9 @@ again:
 	 int val = 1;
 
 	 conn[i].localfd = localfd;
+	 conn[i].lbuf = Malloc(sndbufsz);
+	 conn[i].rbuf = Malloc(sndbufsz);
+
 	 if((flags = fcntl(localfd, F_GETFL, 0)) == -1)
 	    err_sys("Connection_add: fcntl F_GETFL error");
 
@@ -61,7 +50,7 @@ again:
 	 if(setsockopt(localfd, SOL_SOCKET, SO_OOBINLINE, &val, sizeof(int)) == -1)
 	    err_sys("Connection_add: Can not set SO_OOBINLINE");
 
-	 conn[i].time = time(NULL) + timeout;
+	 conn[i].time = time(NULL) + timeout-1;
 	 maxfd = MAX(maxfd, conn[i].localfd);
 
 	 if(conn_size == max_simul_con)
@@ -70,7 +59,9 @@ again:
 	 return i;
       }
    }
-   
+
+   /* Couldn't find any empty connection structures, allocate a new one
+      and do the loop again, it will work this time. */   
    Connection_alloc();
    goto again;
 }
@@ -80,12 +71,15 @@ again:
  */
 
 void Connection_del(Connection *con) {
+   char *ret, *ret2;
 
 #ifdef DEBUG
-   if(debug)
-      err_msg("Connection_del: Deleting Connection structure for localfd=%d, remotefd=%d", con->localfd, con->remotefd);
+   err_msg("Connection_del: Deleting Connection structure for localfd=%d, remotefd=%d", con->localfd, con->remotefd);
 #endif
 
+   aesoplog("Connection from %s to %s ended.", ret = print_sock(con->localfd), ret2 = print_sock(con->remotefd));
+   free(ret); free(ret2);
+
    if(con->localfd) {
       close(con->localfd);
       FD_CLR(con->localfd, &rdset);
@@ -98,27 +92,26 @@ void Connection_del(Connection *con) {
       FD_CLR(con->remotefd, &wrset);
    }
 
-   memset(&con->e, 0, sizeof(ArcfourContext));
-   memset(&con->d, 0, sizeof(ArcfourContext));
-   memset(&con->e2, 0, sizeof(ArcfourContext));
-   memset(&con->d2, 0, sizeof(ArcfourContext));
-
    if(con->dh)
       free(con->dh);
 
-   con->dh = NULL;
-   con->status = ST_NOTUSED;
-   memset(&con->init, 0, sizeof(conninit));
-   con->localfd = 0;
-   con->remotefd = 0;
-   con->lpos = 0;
-   con->rpos = 0;
-   con->l_oobused = 0;
-   con->r_oobused = 0;
+   if(con->init)
+      free(con->init);
 
-   FD_SET(lfd, &rdset);		/* We just deleted a Connection structure there must be one avaiable to accept
-				   a new connection on.
-				 */
+   if(con->l_oobpos)
+      free(con->l_oobpos);
+
+   if(con->r_oobpos)
+      free(con->r_oobpos);
+
+   free(con->lbuf);
+   free(con->rbuf);
+
+   memset(con, 0, sizeof(Connection));
+
+   /* We just deleted a Connection structure there must be one avaiable to accept
+      a new connection on. */
+   FD_SET(lfd, &rdset);
 }
 
 /* Connection_process reads in the conninit structure and checks the magic value.
@@ -131,27 +124,25 @@ void Connection_process(Connection *con) {
    
    int n, sd, max;
    int val = 1;
+   char *tmp, *ret, *ret2;
    long flags;
    conninit init;
    struct sockaddr *sin;
    socklen_t sinlen;   
 
-   if(con->init.control & CNTRL_ENDPO) {	/* 2nd time visit as endpoint */
+   if(con->control & CNTRL_ENDPO) {	/* 2nd time visit as endpoint */
        FD_CLR(con->localfd, &rdset);
-       init = con->init;
+       init = *(con->init);
    } else {
       n = con->lpos;
-      max = sizeof(conninit) - n;
+      max = ENC_CONN_SIZE - n;
 
       Readdata(con, FROM_LOCAL, max);
 
-      if(con->lpos < sizeof(conninit)) {
-
+      if(con->lpos < ENC_CONN_SIZE) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_process: Read in %d bytes of conninit structure from localfd=%d", con->lpos - n, con->localfd);
+	 err_msg("Connection_process: Read in %d bytes of conninit structure from localfd=%d", con->lpos - n, con->localfd);
 #endif
-
 	 FD_CLR(con->remotefd, &wrset);	/* Readdata() set con->remotefd on in wrset */
 	 return;
       }
@@ -160,45 +151,71 @@ void Connection_process(Connection *con) {
       FD_CLR(con->remotefd, &wrset);	/* Readdata() set con->remotefd on in wrset */
 
 #ifdef DEBUG
-      if(debug)
-	 err_msg("Connection_process: Read in conninit structure, processing for localfd=%d", con->localfd);
+      err_msg("Connection_process: Read in conninit structure, processing for localfd=%d", con->localfd);
 #endif
 
-      memcpy(&init, con->lbuf, sizeof(conninit));
-      memmove(con->lbuf, con->lbuf + sizeof(conninit), con->lpos - sizeof(conninit));
-      con->lpos -= sizeof(conninit);
+      dec_conninit(&ServerPrivateKey, con->lbuf, (unsigned char *)&init);
+      memmove(con->lbuf, con->lbuf + ENC_CONN_SIZE, con->lpos - ENC_CONN_SIZE);
+
+      con->lpos -= ENC_CONN_SIZE;
  
       if(ntohl(init.magic) != MAGIC) {
 
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_process: init.magic = 0x%x", init.magic);
+	 err_msg("Connection_process: init.magic = 0x%x", init.magic);
 #endif
 
 	 Connection_del(con);
 	 return;
       }
 
-      con->init = init;
-      con->init.control = ntohs(con->init.control);
+      printf("init.authdata = %s\n", init.authdata);
 
-      if((!ipv6support) && (con->init.control & CNTRL_INET6)) {
+      if(AuthData != NULL) {
+	    char *ret;
+	 tmp = MD5Hash((unsigned char *)init.authdata);
+	 if(strcmp(AuthData, tmp)) {
+#ifdef DEBUG
+	    err_msg("Connection_process: AuthData from localfd=%d is invalid!", con->localfd);
+#endif
+	    aesoplog("Authentication for %s failed.", ret = print_sock(con->localfd));
+	    free(ret);
+	    Connection_del(con);
+	    free(tmp);
+	    return;
+	 }
+	 free(tmp);
+#ifdef DEBUG
+	 err_msg("Connection_process: AuthData from localfd=%d is valid", con->localfd);
+#endif
+	 aesoplog("%s authenticated succesfully.", ret = print_sock(con->localfd));
+	 free(ret);
+      }
+
+      con->init = Malloc(sizeof(conninit));
+      memcpy(con->init, &init, sizeof(conninit));
+      con->init->control = ntohs(con->init->control);
+      con->control = con->init->control;
+      con->init->pkey.bits =  ntohl(con->init->pkey.bits);
+
+      if((!ipv6support) && (con->control & CNTRL_INET6)) {
+#ifdef DEBUG
 	 err_msg("Connection_process: Got unsupported IPv6 request on fd=%d", con->localfd);
+#endif
 	 Connection_del(con);
 	 return;
       }
 
-      if(con->init.control & CNTRL_ENDPO) {	/* Endpoint */
+      if(con->control & CNTRL_ENDPO) {	/* Endpoint */
 	 Connection_DH_init(con, AS_ENDPOINT);
 	 return;
       }
    }
 
    if((sin = w_setsockaddr(&init.targetip, init.targetport, &sinlen,
-			   con->init.control & CNTRL_INET6)) == NULL) {
+			   con->control & CNTRL_INET6)) == NULL) {
 #ifdef DEBUG
-      if(debug)
-	 err_msg("Connection_process: w_setsockaddr returned NULL");
+      err_msg("Connection_process: w_setsockaddr returned NULL");
 #endif
 
       Connection_del(con);
@@ -206,15 +223,15 @@ void Connection_process(Connection *con) {
    }
 
 #ifdef DEBUG
-   if(debug)
-      err_msg("Connection_process: Connection request for localfd=%d to %s", con->localfd, w_sock_ntop(sin));
+   err_msg("Connection_process: Connection request for localfd=%d to %s", con->localfd, ret = w_sock_ntop(sin));
+   free(ret);
 #endif
-
+   aesoplog("%s wants to connect to %s", ret = print_sock(con->localfd), ret2 = w_sock_ntop(sin));
+   free(ret); free(ret2);
    if((sd = socket(sin->sa_family, SOCK_STREAM, 0)) == -1) {
       if(errno == EAFNOSUPPORT) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_process: No AF support for localfd=%d", con->localfd);
+	 err_msg("Connection_process: No AF support for localfd=%d", con->localfd);
 #endif
 	 free(sin);
 	 Connection_del(con);
@@ -233,14 +250,17 @@ void Connection_process(Connection *con) {
    if(setsockopt(sd, SOL_SOCKET, SO_OOBINLINE, &val, sizeof(int)) == -1)
       err_sys("Connection_add: Can not set SO_OOBINLINE");
 
-   if((con->init.control & CNTRL_ENDPO) || (con->init.control & CNTRL_ALONE)) {
-      if((con->init.control & CNTRL_APORT) || (con->init.control & CNTRL_MPORT)) {
-	 int n = con->init.control & CNTRL_APORT ? 1 : 2;
-	 if(ntohs(con->init.data) < sp_base) {
+   if((con->control & CNTRL_ENDPO) || (con->control & CNTRL_ALONE)) {
+      if((con->control & CNTRL_APORT) || (con->control & CNTRL_MPORT)) {
+	 char *ret;
+	 int n = con->control & CNTRL_APORT ? 1 : 2;
+	 aesoplog("%s requested %s sourceport specification for port %d.", ret = print_sock(con->localfd),
+	      n == 1 ? "advisory" : "mandatory", ntohs(init.data));
+	 free(ret);
+	 if(ntohs(init.data) < sp_base) {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_process: %s sourceport specification (%d) is not allowed",
-			n == 1 ? "Advisory" : "Mandatory", ntohs(con->init.data));
+	    err_msg("Connection_process: %s sourceport specification (%d) is not allowed",
+		     n == 1 ? "Advisory" : "Mandatory", ntohs(init.data));
 #endif   
 	    if(n == 1) {
 	       goto rest;
@@ -254,11 +274,10 @@ void Connection_process(Connection *con) {
 	    char data[16];
 
 	    memset(data, 0, 16);
-	    if((s = w_setsockaddr(data, con->init.data, &slen,
-			   con->init.control & CNTRL_INET6)) == NULL) {
+	    if((s = w_setsockaddr(data, init.data, &slen,
+			   con->control & CNTRL_INET6)) == NULL) {
 #ifdef DEBUG
-	       if(debug)
-		  err_msg("Connection_process: w_setsockaddr returned NULL");
+	       err_msg("Connection_process: w_setsockaddr returned NULL");
 #endif
 	       if(n == 1) {
 		  goto rest;
@@ -270,16 +289,14 @@ void Connection_process(Connection *con) {
 
 	    if(setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) == -1) {
 #ifdef DEBUG
-	       if(debug)
-		  err_ret("Connection_process: Could not set SO_REUSEADDR");
+	       err_ret("Connection_process: Could not set SO_REUSEADDR");
 #endif
 	    }
 
 	    if(bind(sd, s, slen) == -1) {
 #ifdef DEBUG
-	       if(debug)
-		  err_ret("Connection_process: %s sourceport(%d) bind() failed",
-			n == 1 ? "Advisory" : "Mandatory", ntohs(con->init.data));
+	       err_ret("Connection_process: %s sourceport(%d) bind() failed",
+			n == 1 ? "Advisory" : "Mandatory", ntohs(init.data));
 #endif
 	       if(n == 1) {
 		  goto rest;
@@ -289,9 +306,8 @@ void Connection_process(Connection *con) {
 	       }
 	    }
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_process: %s sourceport(%d) succesfully bound",
-			n == 1 ? "Advisory" : "Mandatory", ntohs(con->init.data));
+	    err_msg("Connection_process: %s sourceport(%d) succesfully bound",
+		     n == 1 ? "Advisory" : "Mandatory", ntohs(init.data));
 #endif
 	 }
       }
@@ -299,6 +315,12 @@ void Connection_process(Connection *con) {
 
 rest:
 
+   if(!(con->control & CNTRL_CHAIN)) {
+      memset(con->init, 0, sizeof(conninit));
+      free(con->init);
+      con->init = NULL;
+   }
+
    if((n = connect(sd, sin, sinlen)) == -1) {
       if(errno != EINPROGRESS) {
 	 err_ret("Connection_process: connect didnt return EINPROGRESS");
@@ -316,8 +338,7 @@ rest:
    }
 
 #ifdef DEBUG
-   if(debug)
-      err_msg("Connection_process: Processed localfd=%d, remotefd=%d", con->localfd, sd);
+   err_msg("Connection_process: Processed localfd=%d, remotefd=%d", con->localfd, sd);
 #endif
 
    free(sin);
@@ -327,55 +348,61 @@ rest:
    FD_SET(sd, &wrset);
 }
 
+/* Connection_DH_init() does server side DH initialisation. It stores it's part
+   of the protocol in the connection buffer. Subsequent calls of
+   Connection_DH_senddata() write it out later. If 'how' is AS_ENDPOINT the
+   DH data will also be encrypted as it goes over the already setup encrypted link
+*/
 void Connection_DH_init(Connection *con, int how) {
    R_RANDOM_STRUCT RandomStruct;
  
    InitRandomStruct(&RandomStruct);
 
    if((con->dh = InitDH(&RandomStruct, 1, &ServerPrivateKey)) == NULL) {
+#ifdef DEBUG
       err_msg("Connection_DH_init: InitDH() failed");
+#endif
       Connection_del(con);
       return;
    }
    
+   /* Initialise buffer with server side DH data which will be written
+      out by Writedata(). If the server does DH back with the user over
+      an existing encrypted link the buffer must be encrypted. The next
+      if/else statements handle that. */
    memcpy(con->rbuf, con->dh->publicValue, DH_VAL_LEN);
    memcpy(con->rbuf+DH_VAL_LEN, con->dh->signature, DH_SIGNATURE_LEN);
 
    if(how == AS_NORMAL) {
       con->status = ST_DHSEND1;
       FD_SET(con->localfd, &wrset);
-
 #ifdef DEBUG
-      if(debug)
-	 err_msg("Connection_DH_init: Generated DH data for localfd=%d", con->localfd);
+      err_msg("Connection_DH_init: Generated DH data for localfd=%d", con->localfd);
 #endif
-
    } else if(how == AS_CHAIN) {
       con->status = ST_DHSEND2;
       FD_SET(con->remotefd, &wrset);
-
 #ifdef DEBUG
-      if(debug)
-	 err_msg("Connection_DH_init: Generated DH data for remotefd=%d", con->remotefd);
+      err_msg("Connection_DH_init: Generated DH data for remotefd=%d", con->remotefd);
 #endif
-
    } else if(how == AS_ENDPOINT) {
       con->status = ST_DHSEND3;
       con->rpos = DH_DATA_LEN;
       arcfour_encrypt(&con->e, con->rbuf, con->rbuf, DH_DATA_LEN);
       FD_SET(con->localfd, &wrset);
-
 #ifdef DEBUG
-      if(debug)
-	 err_msg("Connection_DH_init: Generated DH data as endpoint for localfd=%d", con->localfd);
+      err_msg("Connection_DH_init: Generated DH data as endpoint for localfd=%d", con->localfd);
 #endif
    }
 }
 
+/* Connection_DH_senddata() will be called subsequently until all DH data has been
+   written out. Once that's done it will change the status of the connection 
+   structure appropriately. */
+
 void Connection_DH_senddata(Connection *con, int to) {
-   int n;
+   int n, fd;
    char *name;
-   int fd;
 
    if(to == TO_LOCAL) {
       name = "localfd";
@@ -384,12 +411,9 @@ void Connection_DH_senddata(Connection *con, int to) {
 	 Writedata(con, TO_LOCAL);
 	 FD_CLR(con->remotefd, &rdset);	/* Was set by Writedata() */
 	 if(!con->rpos) {
-
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_DH_senddata: Send out all server DH data as endpoint to %s=%d", name, fd);
+	    err_msg("Connection_DH_senddata: Send out all server DH data as endpoint to %s=%d", name, fd);
 #endif
-
 	    con->status = ST_DHRECV3;
 	    FD_CLR(con->localfd, &wrset);
 	    FD_SET(con->localfd, &rdset);
@@ -405,21 +429,15 @@ void Connection_DH_senddata(Connection *con, int to) {
 
    switch(n) {
       case -1:
-
 #ifdef DEBUG
-	 if(debug)
-	    err_ret("Connection_DH_senddata: write error to %s=%d", name, fd);
+	 err_ret("Connection_DH_senddata: write error to %s=%d", name, fd);
 #endif
-
 	 Connection_del(con);
 	 break;
       case 0: 	/* Huh?, when does this happen? */
-
 #ifdef DEBUG
-	 if(debug)
-	    err_ret("Connection_DH_senddata: write to %s=%d returned zero", name, fd);
+	 err_ret("Connection_DH_senddata: write to %s=%d returned zero", name, fd);
 #endif
-
 	 Connection_del(con);
 	 break;
       default:
@@ -434,26 +452,27 @@ void Connection_DH_senddata(Connection *con, int to) {
 	       FD_SET(con->remotefd, &rdset);
 	    }
 	    con->rpos = 0;
-
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_DH_senddata: Send out all server DH data to %s=%d", name, fd);
+	    err_msg("Connection_DH_senddata: Send out all server DH data to %s=%d", name, fd);
 #endif
-
 	 } else {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_DH_senddata: Send out %d bytes of DH data to %s=%d", n - con->rpos, name, fd);
+	    err_msg("Connection_DH_senddata: Send out %d bytes of DH data to %s=%d", n - con->rpos, name, fd);
 #endif
 	    con->rpos += n;
 	 }
    }
 }
+
+/* Connection_DH_readdata() will read all DH data from the other side
+   in subsequent calls. As soon as it has read all data it will be passed
+   to Connection_DH_process(). If 'from' is  FROM_LOCAL and the status
+   of the structure is ST_DHRECV3, it will use Readdata() as the DH data
+   must also get decrypted. */
       
 void Connection_DH_readdata(Connection *con, int from) {
-   int n;
+   int n, fd;
    char *name;
-   int fd;
 
    if(from == FROM_LOCAL) {
       name = "localfd";
@@ -464,8 +483,7 @@ void Connection_DH_readdata(Connection *con, int from) {
 	 FD_CLR(con->remotefd, &wrset);	/* Set by Readdata() */
 	 if(con->lpos == DH_DATA_LEN) {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_DH_readdata: Read in all client DH data as endpoint from %s=%d", name, fd);
+	    err_msg("Connection_DH_readdata: Read in all client DH data as endpoint from %s=%d", name, fd);
 #endif
 	    con->lpos = 0;
 	    Connection_DH_process(con, AS_ENDPOINT);
@@ -473,7 +491,7 @@ void Connection_DH_readdata(Connection *con, int from) {
 	 return;
       }
       n = read(con->localfd, (con->lbuf + con->lpos), (DH_DATA_LEN - con->lpos));
-   } else {
+   } else {	/* FROM_REMOTE	*/
       name = "remotefd";
       fd = con->remotefd;
       n = read(con->remotefd, (con->lbuf + con->lpos), (DH_DATA_LEN - con->lpos));
@@ -482,15 +500,13 @@ void Connection_DH_readdata(Connection *con, int from) {
    switch(n) {
       case -1:
 #ifdef DEBUG
-	 if(debug)
-	    err_ret("Connection_DH_readdata: read error from %s=%d", name, fd);
+	 err_ret("Connection_DH_readdata: read error from %s=%d", name, fd);
 #endif
 	 Connection_del(con);
 	 break;
       case 0:
 #ifdef DEBUG
-	 if(debug)
-	    err_ret("Connection_DH_readdata: premature end of file from %s=%d", name, fd);
+	 err_ret("Connection_DH_readdata: premature end of file from %s=%d", name, fd);
 #endif
 	 Connection_del(con);
 	 break;
@@ -498,8 +514,7 @@ void Connection_DH_readdata(Connection *con, int from) {
 	 con->lpos += n;
 	 if(con->lpos == DH_DATA_LEN) {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_DH_readdata: Read in all client DH data from %s=%d", name, fd);
+	    err_msg("Connection_DH_readdata: Read in all client DH data from %s=%d", name, fd);
 #endif
 	    con->lpos = 0;
 	    if(from == FROM_LOCAL)
@@ -509,25 +524,31 @@ void Connection_DH_readdata(Connection *con, int from) {
 	    return;
 	 } else {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_DH_readdata: Read in %d bytes of DH data from %s=%d", n, name, fd);
+	    err_msg("Connection_DH_readdata: Read in %d bytes of DH data from %s=%d", n, name, fd);
 #endif
 	 }
    }
    return;
 }
 
+/* Connection_DH_process() computes the key from the DH parameters.
+   And initialises the appropriate ciphers with it.
+ */
+
 void Connection_DH_process(Connection *con, int how) {
    unsigned char key[64+1];	/* 512 bit key is exchanged */
+   R_RSA_PUBLIC_KEY *Pub = NULL;
+
+   if(how == AS_CHAIN) {
+      con->dh->server = 0;
+      Pub = &con->init->pkey;
+   }
 
    memset(key, 0, 65);
-   if(ComputeDH(con->lbuf, con->dh, NULL, key) != 0) {
-
+   if(ComputeDH(con->lbuf, con->dh, Pub, key) != 0) {
 #ifdef DEBUG
-      if(debug)
-	 err_msg("Connection_DH_process: ComputeDH failed");
+      err_msg("Connection_DH_process: ComputeDH failed");
 #endif
-
       Connection_del(con);
       return;
    }
@@ -545,33 +566,24 @@ void Connection_DH_process(Connection *con, int how) {
    con->dh = NULL;
 
    if(how == AS_NORMAL) {
-      if(con->init.control & CNTRL_ENDPO) {
-	 arcfour_init(&con->e2, key, 64);
-	 con->d2 = con->e2;
-	 FD_CLR(con->localfd, &rdset);
-	 Connection_process(con);
-	 return;
-      } else {
-	 arcfour_init(&con->e, key, 64);
-	 con->d = con->e;	/* Saves a bit of processing time */
-   
-	 FD_CLR(con->localfd, &wrset);
-	 FD_SET(con->localfd, &rdset);
-	 con->status = ST_WAITINF;
-	 con->time = time(NULL) + timeout;
-      }
+      arcfour_init(&con->e, key, 64);
+      con->d = con->e;	/* Saves a bit of processing time */
+      FD_CLR(con->localfd, &wrset);
+      FD_SET(con->localfd, &rdset);
+      con->status = ST_WAITINF;
+      con->time = time(NULL) + timeout;
    } else if(how == AS_CHAIN) {
       arcfour_init(&con->e2, key, 64);
       con->d2 = con->e2;
-      
       FD_SET(con->localfd, &rdset);
       con->status = ST_CONNECT;
+      memset(con->init, 0, sizeof(conninit));
+      free(con->init);
+      con->init = NULL;
    } else if(how == AS_ENDPOINT) {
       arcfour_init(&con->e2, key, 64);
       con->d2 = con->e2;
-
       Connection_process(con);
-      return;
    }
 }
 
@@ -580,32 +592,28 @@ void Connection_eof(Connection *con, int who) {
    if(who == EOF_LOCAL) {
       if(con->status == ST_FINISHL) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_eof: Wrote out all remaining data to remotefd=%d, closing end for writing",
-		     con->remotefd);
+	 err_msg("Connection_eof: Wrote out all remaining data to remotefd=%d, closing end for writing",
+		  con->remotefd);
 #endif
 	 shutdown(con->remotefd, SHUT_WR);
       } else if(con->status == ST_FINISHR) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_eof: Both localfd=%d and remotefd=%d are finished, deleting structure",
-		     con->localfd, con->remotefd);
+	 err_msg("Connection_eof: Both localfd=%d and remotefd=%d are finished, deleting structure",
+		  con->localfd, con->remotefd);
 #endif
 	 Connection_del(con);
 	 return;
       } else {
 	 if(con->lpos) {	/* We have data left to write out */
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_eof: EOF for localfd=%d, %d bytes left to write out, entering ST_FINISHL state",
-			con->localfd, con->lpos);
+	    err_msg("Connection_eof: EOF for localfd=%d, %d bytes left to write out, entering ST_FINISHL state",
+		     con->localfd, con->lpos);
 #endif
 	    FD_SET(con->remotefd, &wrset);
 	 } else {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_eof: EOF for localfd=%d, entering ST_FINISHL state and closing other end for writing",
-			 con->localfd);
+	    err_msg("Connection_eof: EOF for localfd=%d, entering ST_FINISHL state and closing other end for writing",
+		     con->localfd);
 #endif
 	    shutdown(con->remotefd, SHUT_WR);
 	 }
@@ -619,32 +627,28 @@ void Connection_eof(Connection *con, int who) {
    } else if(who == EOF_REMOT) {
       if(con->status == ST_FINISHR) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_eof: Wrote out all remaining data to localfd=%d, closing end for writing",
-		     con->localfd);
+	 err_msg("Connection_eof: Wrote out all remaining data to localfd=%d, closing end for writing",
+		  con->localfd);
 #endif
 	 shutdown(con->localfd, SHUT_WR);
       } else if(con->status == ST_FINISHL) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Connection_eof: Both localfd=%d and remotefd=%d are finished, deleting structure",
-		     con->localfd, con->remotefd);
+	 err_msg("Connection_eof: Both localfd=%d and remotefd=%d are finished, deleting structure",
+		  con->localfd, con->remotefd);
 #endif
 	 Connection_del(con);
 	 return;
       } else {
 	 if(con->rpos) {        /* We have data left to write out */
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_eof: EOF for remotefd=%d, %d bytes left to write out, entering ST_FINISHR state",
-			con->remotefd, con->rpos);
+	    err_msg("Connection_eof: EOF for remotefd=%d, %d bytes left to write out, entering ST_FINISHR state",
+		     con->remotefd, con->rpos);
 #endif
 	    FD_SET(con->localfd, &wrset);
 	 } else {
 #ifdef DEBUG
-	    if(debug)
-	       err_msg("Connection_eof: EOF for remotefd=%d, entering ST_FINISHR state and closing other end for writing",
-			 con->remotefd);
+	    err_msg("Connection_eof: EOF for remotefd=%d, entering ST_FINISHR state and closing other end for writing",
+		     con->remotefd);
 #endif
 	    shutdown(con->localfd, SHUT_WR);
 	 }
@@ -657,3 +661,20 @@ void Connection_eof(Connection *con, int who) {
       }
    }
 }
+
+void dec_conninit(R_RSA_PRIVATE_KEY *pkey, unsigned char *data, unsigned char *dest) {
+   int i;
+   int ret;
+
+   for(i = ENC_CONN_SIZE; i > 0; i -= 128) {
+      unsigned int outputlen, inputlen = 128;
+      
+      if((ret = RSAPrivateDecrypt(dest, &outputlen, data, inputlen, pkey)) != 0) {
+	 err_msg("dec_conninit: RSAPrivateDecrypt() failure: %d", ret);
+	 return;
+      }
+
+      dest += 117;
+      data += 128;
+  }
+}
