@@ -46,6 +46,8 @@ typedef struct _conninit {
    uint16_t     targetport;     /* Network Order */
    uint16_t     control;        /* Control codes for proxy in network order */
    uint16_t	data;		/* Data field (used by APORT, MPORT, etc) Network Order	*/
+   uint8_t      authdata[40];
+   R_RSA_PUBLIC_KEY pkey;       /* pkey.bits in Network Order   */
 #ifndef __GNUC__
  } conninit;
  #pragma pack(0)
@@ -53,6 +55,8 @@ typedef struct _conninit {
  } __attribute__ ((packed)) conninit;
 #endif
 
+#define ENC_CONN_SIZE   (3 * 128)       /* Size of encrypted conninit structure */
+
 typedef struct connection {
    int localfd;		/* FD representing user-proxy link */
    int remotefd;	/* FD representing proxy-target link */
@@ -73,7 +77,8 @@ typedef struct connection {
    ArcfourContext d;	/* RC4 context used for decryption */
    ArcfourContext e2;	/* 2nd RC4 context for encryption in chain or endpoint mode */
    ArcfourContext d2;	/* 2nd RC4 context for encryption in chain or endpoint mode */
-   conninit init;	/* Copy of conninit structure for this connection */
+   int control;		/* Copy of control field of control header for this connection.	*/
+   conninit *init;	/* Temporary copy of the conninit structure for this connection.*/
 } Connection;
 
 void Connection_alloc(void);
@@ -85,5 +90,6 @@ void Connection_DH_senddata(Connection *con, int to);
 void Connection_DH_readdata(Connection *con, int from);
 void Connection_DH_process(Connection *con, int how);
 void Connection_eof(Connection *con, int who);
+void dec_conninit(R_RSA_PRIVATE_KEY *pkey, unsigned char *data, unsigned char *dest);
 
 #endif
