@@ -3,13 +3,14 @@
 void initialise(int argc, char **argv, int fd[2]) {
    char *chrootdir = NULL;
    char *keyfile = NULL;
+   char *conffile = NULL;
    char *ptr;
    int uid = -1, gid = -1;
    int o, udp_probe = 1;
    long flags;
 
    opterr = 0;
-   while((o = getopt(argc, argv, "?hvdel:c:u:g:s:k:t:b:r:")) != EOF) {
+   while((o = getopt(argc, argv, "?hvdea:l:n:o:c:u:g:s:k:t:b:r:")) != EOF) {
       switch(o) {
 	 case 'h':
 	 case '?':
@@ -29,13 +30,21 @@ void initialise(int argc, char **argv, int fd[2]) {
 	 case 'e':
 	    udp_probe = 0;
 	    break;
-	 case 'l':
+	 case 'n':
 	    max_simul_con = atoi(optarg);
 	    break;
+	 case 'a':
+	    AuthData = strdup(optarg);
+	    break;
+	 case 'l':
+	    do_log = 1;
+	    LogFile = strdup(optarg);
+	    break;
+	 case 'o':
+	    conffile = strdup(optarg);
+	    break;
 	 case 'c':
-	    if((chrootdir = malloc(strlen(optarg)+1)) == NULL)
-	       err_sys("memory allocation error");
-	    strcpy(chrootdir, optarg);
+	    chrootdir = strdup(optarg);
 	    break;
 	 case 'u':
 	    uid = atoi(optarg);
@@ -47,9 +56,7 @@ void initialise(int argc, char **argv, int fd[2]) {
 	    sndbufsz = atoi(optarg);
 	    break;
 	 case 'k':
-	    if((keyfile = malloc(strlen(optarg)+1)) == NULL)
-	       err_sys("memory allocation error");
-	    strcpy(keyfile, optarg);
+	    keyfile = strdup(optarg);
 	    break;
 	 case 't':
 	    timeout = atoi(optarg);
@@ -58,9 +65,7 @@ void initialise(int argc, char **argv, int fd[2]) {
 	    sp_base = atoi(optarg);
 	    break;
 	 case 'r':
-	    if((seedfile = malloc(strlen(optarg)+1)) == NULL)
-	       err_sys("memory allocation error");
-	    strcpy(seedfile, optarg);
+	    seedfile = strdup(optarg);
 	    break;
 	 default:
 	    fprintf(stderr, "Invalid option: -%c\n", o);
@@ -69,12 +74,80 @@ void initialise(int argc, char **argv, int fd[2]) {
       }
    }
 
+   if(!conffile) {
+      if(!access(AESOPD_CONF_FILE, F_OK))
+	 conf_parse(AESOPD_CONF_FILE);
+      else if(!access(GLOBAL_AESOPD_CONF_FILE, F_OK))
+	 conf_parse(GLOBAL_AESOPD_CONF_FILE);
+   } else {
+      conf_parse(conffile);
+      free(conffile);
+   }
+
+   /* Get parameters from configuration file if not overridden by commandline	*/
+
+   if(!LogFile) {
+      if((LogFile = getvalue("LogFile")) != NULL)
+	 do_log = 1;
+   }
+
+   if(!chrootdir)
+      chrootdir = getvalue("ChrootDir");
+
+   if(!uid) {
+      char *tmp = getvalue("User");
+      if(tmp)
+	 uid = atoi(tmp);
+   }
+
+   if(!gid) {
+      char *tmp = getvalue("Group");
+      if(tmp)
+	 gid = atoi(tmp);
+   }
+
+   if(getvalue("MaxSimulConn"))
+      max_simul_con = atoi(getvalue("MaxSimulConn"));
+
+   if(!sndbufsz) {
+      char *tmp = getvalue("BufferSize");
+      if(tmp)
+	 sndbufsz = atoi(tmp);
+   }
+
+   if(!keyfile)
+      keyfile = getvalue("PrivateKeyFile");
+
+   if(!AuthData) {
+      char *tmp = getvalue("AuthString");
+      if(tmp)
+	 AuthData = strdup(tmp);
+   }
+
+   if(timeout != -1) {
+      char *tmp = getvalue("TimeOut");
+      if(tmp)
+	 timeout = atoi(tmp);
+   } else
+      timeout = TIMEOUT;
+
+   if(!sp_base) {
+      char *tmp = getvalue("SourcePortBase");
+      if(tmp)
+	 sp_base = atoi(tmp);
+   }
+
+   if(!seedfile) {
+      char *tmp = getvalue("SeedFile");
+      if(tmp)
+	 seedfile = strdup(tmp);
+      else
+	 seedfile = RANDOM_SEED_FILE;
+   }
+
    if(LoadPrivateKey(keyfile ? keyfile : PRIVATE_KEY_FILE, &ServerPrivateKey) != 0)
       exit(-1);         /* LoadPrivateKey prints error message */
 
-   if(keyfile)
-      free(keyfile);
-
    fd[1] = 0;
 
    if((argc - optind) > 0) {
@@ -132,10 +205,11 @@ void initialise(int argc, char **argv, int fd[2]) {
 	 exit(0);
       } else {
 	 setsid();
-	 openlog(argv[0], LOG_PID, LOG_USER);
 	 daemon_proc = 1;
       }
    }
+   aesoplog("Aesopd started.");
+   conf_cleanup();
    signal(SIGPIPE, sigpipe);
 }
 
@@ -148,23 +222,26 @@ void sigpipe(int dummy) {
 }
 
 void usage(char *name) {
-   fprintf(stderr, "aesopd: Advanced Encrypted Stackable Open Proxy v0.1\n");
+   fprintf(stderr, "aesopd: Advanced Encrypted Stackable Open Proxy v%s\n", VERSION);
    fprintf(stderr, "usage: %s [options] [[listenhost[/v4|/v6]#]port]\n", name);
    fprintf(stderr, "where options are:\n");
-   fprintf(stderr, "\t-h\t\t: Show this help.\n");
+   fprintf(stderr, "\t-a string\t: Specify the authentication string this proxy requires.\n");
+   fprintf(stderr, "\t-b base\t\t: Set sourceport specification base. base > 65535 to disallow.\n");
+   fprintf(stderr, "\t-c dir\t\t: Chroot daemon to dir.\n");
 #ifdef DEBUG
    fprintf(stderr, "\t-d\t\t: Run the daemon in debug mode.\n");
-#endif
-   fprintf(stderr, "\t-c dir\t\t: Chroot daemon to dir.\n");
-   fprintf(stderr, "\t-u uid\t\t: Run daemon under uid.\n");
-   fprintf(stderr, "\t-g gid\t\t: Run daemon under gid.\n");
    fprintf(stderr, "\t-e\t\t: Disable support for udp probing.\n");
-   fprintf(stderr, "\t-l num\t\t: Maximum simultaneous connections.\n");
-   fprintf(stderr, "\t-s size\t\t: Set internal buffersize.\n");
+#endif
+   fprintf(stderr, "\t-g gid\t\t: Run daemon under gid.\n");
+   fprintf(stderr, "\t-h\t\t: Show this help.\n");
    fprintf(stderr, "\t-k file\t\t: Specify private keyfile.\n");
-   fprintf(stderr, "\t-t timeout\t: Set connection timeout.\n");
-   fprintf(stderr, "\t-b base\t\t: Set sourceport specification base. base > 65535 to disallow.\n");
+   fprintf(stderr, "\t-l file\t\t: Log to file.\n");
+   fprintf(stderr, "\t-n num\t\t: Maximum simultaneous connections.\n");
+   fprintf(stderr, "\t-o file\t\t: Read configuration from file.\n");
    fprintf(stderr, "\t-r file\t\t: Specify file to take random seed from.\n");
+   fprintf(stderr, "\t-s size\t\t: Set internal buffersize.\n");
+   fprintf(stderr, "\t-t timeout\t: Set connection timeout.\n");
+   fprintf(stderr, "\t-u uid\t\t: Run daemon under uid.\n");
    fprintf(stderr, "\t-v\t\t: Show version information.\n");
    exit(0);
 }
