@@ -2,7 +2,12 @@
 
 int main(int argc, char **argv) {
    int listenfd, num, i;
+   unsigned int timeoffset;
+   struct sockaddr *cli;
+   socklen_t clilen, clilensave;
    fd_set rdsettmp, wrsettmp;
+   struct timeval tv;
+   time_t currtime;
 
    listenfd = initialise(argc, argv);
 
@@ -11,16 +16,43 @@ int main(int argc, char **argv) {
    FD_SET(listenfd, &rdset);
    maxfd = listenfd;
 
-   for( ; ; ) {
-      rdsettmp = rdset;
-      wrsettmp = wrset;
-
-#ifdef DEBUG
-      if(debug)
-	 err_msg("main: in select()");
+#ifdef IPV6
+   clilensave = sizeof(struct sockaddr_in6);
+#else
+   clilensave = sizeof(struct sockaddr_in);
 #endif
 
-      num = select(maxfd+1, &rdsettmp, &wrsettmp, NULL, NULL);
+   cli = Malloc(clilensave);
+
+   for( ; ; ) {
+      /* Timeout handling */
+
+      currtime = time(NULL);
+      timeoffset = 10000000;    /* Very big timeperiod =) */
+
+      for(i = 0; i < conn_size; i++) {
+	 if(IS_ST_TIMEOUT(conn[i])) {
+	    if(currtime > conn[i].time) {
+#ifdef DEBUG
+	       err_msg("Timeout for localfd=%d", conn[i].localfd);
+#endif
+	       aesoplog("Connection from %s timed out.", print_sock(conn[i].localfd));
+	       Connection_del(&conn[i]);
+	       continue;
+	    } else
+		timeoffset = MIN(timeoffset, (conn[i].time - currtime));
+	 }
+      }
+
+      tv.tv_sec = timeoffset+1;         /* +1 to prevent race conditions */
+      tv.tv_usec = 0;
+
+      rdsettmp = rdset;
+      wrsettmp = wrset;
+#ifdef DEBUG
+      err_msg("main: in select()");
+#endif
+      num = select(maxfd+1, &rdsettmp, &wrsettmp, NULL, &tv);
 
       if(num == -1)
 	 if(errno == EINTR)
@@ -31,35 +63,29 @@ int main(int argc, char **argv) {
 	 continue;              /* Timeout detected */
 
 #ifdef DEBUG
-      if(debug)
-	 err_msg("main: selectloop: select returned %d", num);
+      err_msg("main: selectloop: select returned %d", num);
 #endif
 
       if(FD_ISSET(listenfd, &rdsettmp)) {
 	 int connfd, i;
-	 struct sockaddr *cli;
-#ifdef IPV6
-	 socklen_t clilen = sizeof(struct sockaddr_in6);
-#else
-	 socklen_t clilen = sizeof(struct sockaddr_in);
-#endif
-	 if((cli = malloc(clilen)) == NULL)
-	    err_quit("main: Memory allocation error");
+	 char *ret;
+
+	 clilen = clilensave;
 
 	 if((connfd = accept(listenfd, cli, &clilen)) == -1) {
 	    if(errno == EWOULDBLOCK || errno == ECONNABORTED)
 	       continue;        /* Connection was reset before we could accept */
 	    else
-	       err_sys("Accept error");
+	       continue;
 	 }
 
 #ifdef DEBUG
-	 if(debug)
-	   err_msg("main: accepted connection from %s on fd=%d", w_sock_ntop(cli), connfd);
+	 err_msg("main: accepted connection from %s on fd=%d", w_sock_ntop(cli), connfd);
 #endif
 
-	 free(cli);
-
+	 aesoplog("Accepted connection from %s.", ret = w_sock_ntop(cli));
+	 free(ret);
+	 
 	 i = Connection_add(connfd);
 	 if(!need_desthdr)
 	    Connection_connect(&conn[i]);
@@ -82,9 +108,7 @@ int main(int argc, char **argv) {
 	 }
 
 	 if(FD_ISSET(conn[i].remotefd, &rdsettmp)) {
-	    if(conn[i].status == ST_DHRECV1)
-	       Connection_DH_readdata(&conn[i]);
-	    else if(conn[i].status == ST_DHRECV2)
+	    if(conn[i].status == ST_DHRECV1 || conn[i].status == ST_DHRECV2)
 	       Connection_DH_readdata(&conn[i]);
 	    else if(IS_ST_PROCESS(conn[i]))
 	       Readdata(&conn[i], FROM_REMOTE, 0);
@@ -93,9 +117,7 @@ int main(int argc, char **argv) {
 	 if(FD_ISSET(conn[i].remotefd, &wrsettmp)) {
 	    if(conn[i].status == ST_INPROGR)
 	       CheckConnection(&conn[i]);
-	    else if(conn[i].status == ST_DHSEND1)
-	       Connection_DH_senddata(&conn[i]);
-	    else if(conn[i].status == ST_DHSEND2)
+	    else if(conn[i].status == ST_DHSEND1 || conn[i].status == ST_DHSEND2)
 	       Connection_DH_senddata(&conn[i]);
 	    else if(conn[i].status == ST_SENDINF)
 	       Connection_sendinfo(&conn[i]);
