@@ -6,15 +6,19 @@
 
 #include "connection.h"
 #include "init.h"
-#include "parse.h"
+#include "parser.h"
+#include "conf.h"
 #include "probe.h"
 #include "control.h"
 #include "io.h"
+#include "log.h"
 
+extern int lfd;
 extern Connection *conn;
 extern int conn_size, maxfd, debug, sndbufsz;
 extern int timeout;
 extern conninit *cntrl;
+extern unsigned char *ctrlheaders;
 extern int cntrl_num;
 extern int skip_ok, probe_all;
 extern int skip_verification;
@@ -24,6 +28,12 @@ extern fd_set rdset, wrset;
 extern struct sockaddr *proxyhost;
 extern socklen_t proxyhostlen;
 extern int ipv6support;
-extern R_RSA_PUBLIC_KEY ServerPublicKey;
+extern R_RSA_PUBLIC_KEY FirstPublicKey;
+extern R_RSA_PUBLIC_KEY EndpointPublicKey;
+extern char *AuthData;
+extern int max_simul_con;
+extern int do_log;
+extern char *LogFile;
+extern int linecount;
 
 #endif
