@@ -14,8 +14,7 @@
 #define ST_RD_DEST	10	/* Reading in destination header	*/
 
 
-#define IS_ST_TIMEOUT(c)	(((c).status == ST_DHSEND1) || ((c).status == ST_DHRECV1) || ((c).status == ST_DHSEND2) || \
-				((c).status == ST_DHRECV2))
+#define IS_ST_TIMEOUT(c)	((c).status == ST_RD_DEST)
 #define IS_ST_PROCESS(c)	(((c).status == ST_CONNECT) || ((c).status == ST_FINISHL) || ((c).status == ST_FINISHR))
 
 #define FOR_NORMAL	0
@@ -44,6 +43,8 @@ typedef struct _conninit {
    uint16_t     targetport;     /* Network Order */
    uint16_t     control;        /* Control codes for proxy in network order */
    uint16_t	data;
+   uint8_t	authdata[40];
+   R_RSA_PUBLIC_KEY pkey;       /* pkey.bits in Network Order   */
 #ifndef __GNUC__
  } conninit;
  #pragma pack(0)
@@ -51,10 +52,28 @@ typedef struct _conninit {
  } __attribute__ ((packed)) conninit;
 #endif
 
+#define ENC_CONN_SIZE	(3 * 128)	/* Size of encrypted conninit structure	*/
+
+#ifndef __GNUC__
+ #pragma pack(1)
+#endif
+typedef struct _desthdr {
+   uint32_t     magic;          /* Magic Number: "AESO" in network order */
+   uint8_t      targetip[16];   /* Network Order */
+   uint16_t     targetport;     /* Network Order */
+   uint16_t     control;        /* Control codes for proxy in network order */
+#ifndef __GNUC__
+ } desthdr;
+ #pragma pack(0)
+#else
+ } __attribute__ ((packed)) desthdr;
+#endif
+
 typedef struct connection {
    int localfd;		/* FD representing user-tunnel link */
    int remotefd;	/* FD representing tunnel-proxy link */
    int status;
+   time_t time;         /* Timestamp on when the current Connection structure was added */
    unsigned char *lbuf;	/* Read buffer for localfd */
    unsigned char *rbuf;	/* Read buffer for remotefd */
    int lpos;		/* Read position in localfd read buffer */
