@@ -1,8 +1,9 @@
 #ifndef CONTROL_H
 #define CONTROL_H
 
-void add_proxy(char *proxy, int port, int probe);
+void add_proxy(Proxy *proxy);
 void proxy_final(int dest_included);
 int resolve_proxy(char *proxy, char *dest);
+void enc_conninit(R_RSA_PUBLIC_KEY *pkey, unsigned char *data, unsigned char *dest);
 
 #endif
