@@ -1,25 +1,44 @@
 #include "aesoptunnel.h"
 
+int lfd;                                /* Copy of the main server listen FD            */
 Connection *conn = NULL;                /* Pointer to array of Connection structures    */
 int conn_size = 0;                      /* Number of Connection structures in array     */
 conninit *cntrl = NULL;			/* Pointer to array of conninit structures	*/
+unsigned char *ctrlheaders = NULL;
 int cntrl_num = 0;			/* Number of conninit structures in array	*/
 fd_set rdset, wrset;                    /* Global read and write sets                   */
 int maxfd;                              /* Global maxfd variable used for select()      */
-int debug = 0;                          /* Interactive/debug mode                       */
 int sndbufsz = 0;                       /* Size of TCP Sendbuffer                       */
-R_RSA_PUBLIC_KEY ServerPublicKey;	/* Public RSA key of the proxy			*/
+R_RSA_PUBLIC_KEY FirstPublicKey;	/* Public RSA key of the first proxy		*/
+R_RSA_PUBLIC_KEY EndpointPublicKey;	/* Public RSA key of the endpoint proxy         */
 int probe_all = 0;			/* Do an UDP probe on all proxies		*/
 int skip_ok = 0;			/* May skip proxies in a chain if they are not up	*/
 int skip_verification = 0;		/* Set true if the Server signature does nto have to be verified	*/
-int h_errno;				/* Resolver error var.				*/
+int max_simul_con = MAX_SIMUL_CONNECT;  /* Maximum simultaneous connections             */
 int need_desthdr = 0;			/* Do we need to read in destination host header?	*/
 int sp_port = 0;			/* Specified source port			*/
 int sp_spec = 0;			/* Sourceport specification? 0=no, 1=adv, 2=man	*/
+char *AuthData = NULL;
 struct sockaddr *proxyhost = NULL;	/* First proxy, used for a direct connection.	*/
 socklen_t proxyhostlen = 0;		/* Size of proxyhost				*/
+int timeout  = -1;                 /* Timeout                                      */
 #ifdef IPV6				/* Runtime ipv6 support checking		*/
 int ipv6support = 1;
 #else
 int ipv6support = 0;
 #endif
+
+char *keywords[] = {
+   "ChrootDir",
+   "User",
+   "Group",
+   "MaxSimulConn",
+   "BufferSize",
+   "TimeOut",
+   "SeedFile",
+   "LogFile",
+   "RoutesFile",
+   "Label",
+   "UDPProbe",
+   NULL
+};
