@@ -1,47 +1,54 @@
 #include "aesoptunnel.h"
 
+/* Readdata() tries to read as much bytes from the local or remote fd specified
+   by 'from' as there is free space available in the buffer, but no more than
+   'max' if 'max' is not zero.
+   It checks for Out of Band data and if available it's pushed on the connection
+   structures oob queue. Readdata() decrypts the readed data before it's added
+   depending on the mode the structure is in. Readdata() also modifies the global
+   readset and writeset fd_set's depending on if there's data available to write
+   out or in the situation that the buffer is full.
+*/
 void Readdata(Connection *con, int from, int max) {
-   int n;
-   int free;
-   int fd;
+   int n, count, fd;
    char *name;
+   unsigned char *buf;
 
    if(from == FROM_LOCAL) {
       fd = con->localfd;
       name = "localfd";
-      free = (sndbufsz - con->lpos);
+      count = (sndbufsz - con->lpos);
       if(max)
-	 free = MIN(free, max);
+	 count = MIN(count, max);
       if(sockatmark(con->localfd)) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Readdata: Read OOB data from localfd=%d", con->localfd);
+	 err_msg("Readdata: Read OOB data from localfd=%d", con->localfd);
 #endif
 	 oob_add(&con->l_oobpos, &con->l_oobsize, &con->l_oobused, con->lpos);
       }
-      n = read(con->localfd, (con->lbuf + con->lpos), free);
+      buf = con->lbuf + con->lpos;
    }
    else {
       fd = con->remotefd;
       name = "remotefd";
-      free = (sndbufsz - con->rpos);
+      count = (sndbufsz - con->rpos);
       if(max)
-	 free = MIN(free, max);
+	 count = MIN(count, max);
       if(sockatmark(con->remotefd)) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Readdata: Read OOB data from remotefd=%d", con->remotefd);
+	 err_msg("Readdata: Read OOB data from remotefd=%d", con->remotefd);
 #endif
 	 oob_add(&con->r_oobpos, &con->r_oobsize, &con->r_oobused, con->rpos);
       }
-      n = read(con->remotefd, (con->rbuf + con->rpos), free);
+      buf = con->rbuf + con->rpos;
    }
 
+   n = read(fd, buf, count);
+
    switch(n) {
       case -1:
 #ifdef DEBUG
-	if(debug)
-	   err_ret("Readdata: Read error from %s=%d", name, fd);
+	err_ret("Readdata: Read error from %s=%d", name, fd);
 #endif
 	Connection_del(con);
 	break;
@@ -50,31 +57,28 @@ void Readdata(Connection *con, int from, int max) {
 	break;
       default:
 #ifdef DEBUG
-	if(debug)
-	   err_msg("Readdata: Read %d bytes from %s=%d", n, name, fd);
+	err_msg("Readdata: Read %d bytes from %s=%d", n, name, fd);
 #endif
 	if(from == FROM_LOCAL) {
-	   if(cntrl_num > 1)	/* multiple proxies, encrypting for endpoint	*/
-	      arcfour_encrypt(&con->e2, (con->lbuf + con->lpos), (con->lbuf + con->lpos), n);
-	   arcfour_encrypt(&con->e, (con->lbuf + con->lpos), (con->lbuf + con->lpos), n);
+	   if((cntrl_num > 1) || (need_desthdr && cntrl_num))	/* multiple proxies, encrypting for endpoint	*/
+	      arcfour_encrypt(&con->e2, buf, buf, n);
+	   arcfour_encrypt(&con->e, buf, buf, n);
 	   con->lpos += n;
 	   if(con->lpos == sndbufsz) { /* Receive buffer is full, clear localfd from readset */
 #ifdef DEBUG
-	      if(debug)
-		 err_msg("Readdata: Receive buffer for localfd=%d is full", con->localfd);
+	      err_msg("Readdata: Receive buffer for localfd=%d is full", con->localfd);
 #endif
 	      FD_CLR(con->localfd, &rdset);
 	   }
 	   FD_SET(con->remotefd, &wrset); /* There's data to write */
 	} else {
-	   arcfour_decrypt(&con->d, (con->rbuf + con->rpos), (con->rbuf + con->rpos), n);
-	   if(cntrl_num > 1)	/* multiple proxies, decrypting for endpoint    */
-	      arcfour_decrypt(&con->d2, (con->rbuf + con->rpos), (con->rbuf + con->rpos), n);
+	   arcfour_decrypt(&con->d, buf, buf, n);
+	   if((cntrl_num > 1) || (need_desthdr && cntrl_num))	/* multiple proxies, decrypting for endpoint    */
+	      arcfour_decrypt(&con->d2, buf, buf, n);
 	   con->rpos += n;
 	   if(con->rpos == sndbufsz) {  /* Receive buffer is full, clear remotefd from readset */
 #ifdef DEBUG
-	      if(debug)
-		 err_msg("Readdata: Receive buffer for remotefd=%d is full", con->remotefd);
+	      err_msg("Readdata: Receive buffer for remotefd=%d is full", con->remotefd);
 #endif
 	      FD_CLR(con->remotefd, &rdset);
 	   }
@@ -84,9 +88,15 @@ void Readdata(Connection *con, int from, int max) {
    }
 }
 
+/* Writedata() tries to write the entire buffer contents out to the remote
+   or local fd specified by 'to'. If connection structure is at a OOB position
+   it will send it out as OOB byte. The internal buffers will be shifted by
+   the amount of data written out. And a vector correction on the queue with OOB
+   positions will be done. Writedata() modifies the readset and writeset fd_set's
+   depending on if there is still data left in the buffers or not.
+*/
 void Writedata(Connection *con, int to) {
-   int n;
-   int fd;
+   int n, fd;
    char *name;
 
    if(to == TO_LOCAL) {
@@ -94,8 +104,7 @@ void Writedata(Connection *con, int to) {
       name = "localfd";
       if(con->r_oobused && (con->r_oobpos[0] == 0)) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Writedata: Send 1 byte of OOB data to localfd=%d", con->localfd);
+	 err_msg("Writedata: Send 1 byte of OOB data to localfd=%d", con->localfd);
 #endif
 	 n = send(con->localfd, con->rbuf, 1, MSG_OOB); /* We send 1 byte of OOB data   */
 	 oob_shift(con->r_oobpos, &con->r_oobused);
@@ -107,8 +116,7 @@ void Writedata(Connection *con, int to) {
       name = "remotefd";
       if(con->l_oobused && (con->l_oobpos[0] == 0)) {
 #ifdef DEBUG
-	 if(debug)
-	    err_msg("Writedata: Send 1 byte of OOB data to remotefd=%d", con->remotefd);
+	 err_msg("Writedata: Send 1 byte of OOB data to remotefd=%d", con->remotefd);
 #endif
 	 n = send(con->remotefd, con->lbuf, 1, MSG_OOB); /* We send 1 byte of OOB data  */
 	 oob_shift(con->l_oobpos, &con->l_oobused);
@@ -120,21 +128,18 @@ void Writedata(Connection *con, int to) {
    switch(n) {
       case -1:
 #ifdef DEBUG
-	if(debug)
-	   err_ret("Writedata: Write error to %s=%d", name, fd);
+	err_ret("Writedata: Write error to %s=%d", name, fd);
 #endif
 	Connection_del(con);
 	break;
       case 0:   /* Nothing written? strange.. */
 #ifdef DEBUG
-	if(debug)
-	   err_msg("Writedata: Write to %s=%d returned zero..", name, fd);
+	err_msg("Writedata: Write to %s=%d returned zero..", name, fd);
 #endif
 	 break;
       default:
 #ifdef DEBUG
-	if(debug)
-	   err_msg("Writedata: Wrote %d bytes to %s=%d", n, name, fd);
+	err_msg("Writedata: Wrote %d bytes to %s=%d", n, name, fd);
 #endif
 	if(to == TO_LOCAL) {
 	   con->rpos -= n;
@@ -166,24 +171,21 @@ void Writedata(Connection *con, int to) {
 }
 
 void CheckConnection(Connection *con) {
-   int error = 0;
-   int optlen = sizeof(int);
+   int error = 0, optlen = sizeof(int);
 
    if(getsockopt(con->remotefd, SOL_SOCKET, SO_ERROR, &error, &optlen) == -1)
       err_sys("main: getsockopt SO_ERROR error");
 
    if(error) {   /* connection did not succeed */
 #ifdef DEBUG
-      if(debug)
-	 err_msg("main: Connection for remotefd=%d did not succeed", con->remotefd);
+      err_msg("main: Connection for remotefd=%d did not succeed", con->remotefd);
 #endif
       Connection_del(con);
       return;
    }
 
 #ifdef DEBUG
-   if(debug)
-      err_msg("main: Connection for remotefd=%d did succeed", con->remotefd);
+   err_msg("main: Connection for remotefd=%d did succeed", con->remotefd);
 #endif
 
    FD_CLR(con->remotefd, &wrset);
