@@ -1,13 +1,12 @@
 #include "aesoptunnel.h"
 
 int probe_proxy(char *ip, int port, int inet6) {
-   int num_probes = 3;
-   int sd, result = 1;
+   int num_probes = 3, sd, result = 1;
    struct sockaddr *sa;
    socklen_t sinlen;
 
    if(!ipv6support && inet6) {
-      err_msg("probe_proxy: Warning, host not built with IPv6 support, skipping host");
+      err_msg("probe_proxy: Warning, host not built with IPv6 support, skipping host.");
       return 0;
    }      
 
@@ -15,14 +14,15 @@ int probe_proxy(char *ip, int port, int inet6) {
 
    if((sd = socket(sa->sa_family, SOCK_DGRAM, 0)) == -1) {
       if(errno == EAFNOSUPPORT) {
-	 err_msg("probe_proxy: Warning, host built with IPv6 support, but without IPv6 stack, skipping host");
+	 err_msg("probe_proxy: Warning, host built with IPv6 support, but without IPv6 stack, skipping host.");
 	 free(sa);
-      }
-      err_sys("probe_proxy: Can not create SOCK_DGRAM socket");
+	 return -1;
+      } else
+	 err_sys("probe_proxy: Can not create SOCK_DGRAM socket.");
    }
 
    if(connect(sd, sa, sinlen) == -1)
-      err_sys("Can not connect to dgram socket");
+      err_sys("probe_proxy: Can not connect to dgram socket.");
 
    while(num_probes--) {
       char rep[12];
@@ -40,7 +40,7 @@ int probe_proxy(char *ip, int port, int inet6) {
 	 if(read(sd, rep, 9) == -1)
 	    break;
 	 if(strncmp(rep, "AESO UP", 7)) {
-	    err_msg("Incorrect response on probe");
+	    err_msg("probe_proxy: Incorrect response on probe, skipping host.");
 	    break;
 	 } else {	/* Got correct reply */
 	    result = 0;
