@@ -9,5 +9,8 @@
 #define PRIVATE_KEY_FILE        "Server.prv"
 #define PUBLIC_KEY_FILE         "Server.pub"
 #define RANDOM_SEED_FILE        "/dev/urandom"	/* Default seed file, overridable with -r	*/
-
+#define AESOPD_CONF_FILE	"aesopd.conf"
+#define GLOBAL_AESOPD_CONF_FILE "/etc/"AESOPD_CONF_FILE
+#define AESOPTUNNEL_CONF_FILE	"aesoptunnel.conf"
+#define GLOBAL_AESOPTUNNEL_CONF_FILE "/etc/"AESOPTUNNEL_CONF_FILE
 #endif
