@@ -11,17 +11,16 @@
 #ifndef __GNUC__
  #pragma pack(1)
 #endif
-typedef struct _conninit {
+typedef struct _desthdr {
    uint32_t     magic;          /* Magic Number: "AESO" in network order */
    uint8_t      targetip[16];   /* Network Order */
    uint16_t     targetport;     /* Network Order */
    uint16_t     control;        /* Control codes for proxy in network order */
-   uint16_t	data;
 #ifndef __GNUC__
- } conninit;
+ } desthdr;
  #pragma pack(0)
 #else
- } __attribute__ ((packed)) conninit;
+ } __attribute__ ((packed)) desthdr;
 #endif
 
 extern int (*_r_connect)(int, const struct sockaddr *, socklen_t);
