@@ -2,18 +2,19 @@
 
 char *seedfile = NULL;
 
-unsigned int urand(void)
-{
+unsigned int urand(void) {
    unsigned int dword;
-   FILE *fptr;
+   static FILE *fptr = NULL;
 
-   if ((fptr = fopen(seedfile == NULL ? RANDOM_SEED_FILE : seedfile, "r")) == NULL) return rand();
-   if (fread(&dword, 4, 1, fptr) != 1)
-     {
-	fclose(fptr);
-	return rand();
-     }
-   fclose(fptr);
+   if(fptr == NULL) {
+      if((fptr = fopen(seedfile, "r")) == NULL)
+	 err_quit("Could not open random seed file %s.", seedfile);
+   }
+
+   if(fread(&dword, 4, 1, fptr) != 1) {
+      fclose(fptr);
+      return rand();
+   }
 
    return dword;
 }
@@ -29,7 +30,7 @@ int InitRandomStruct(R_RANDOM_STRUCT *randomStruct) {
    R_RandomInit(randomStruct);
 
    while(1) {
-      R_GetRandomBytesNeeded (&bytesNeeded, randomStruct);
+      R_GetRandomBytesNeeded(&bytesNeeded, randomStruct);
       if(!bytesNeeded) break;
 
       randData = rand();
@@ -76,11 +77,10 @@ int LoadPublicKey(char *file, R_RSA_PUBLIC_KEY *publicKey) {
    FILE *fptr;
    char tempbuf[20]; /* adjust later */
 
-   if((fptr = fopen(file, "r")) == NULL)
-      {
-	 fprintf(stderr, "LoadPublicKey(): Couldn't open %s for reading\n", file);
-	 return -1;
-      }
+   if((fptr = fopen(file, "r")) == NULL) {
+      fprintf(stderr, "LoadPublicKey(): Couldn't open %s for reading\n", file);
+      return -1;
+   }
 
    fgets(tempbuf, 20, fptr);
    if(strncmp(tempbuf, "RSA PUBLIC KEY", 14))
@@ -125,11 +125,10 @@ int LoadPrivateKey(char *file, R_RSA_PRIVATE_KEY *privateKey) {
    FILE *fptr;
    char tempbuf[20]; /* adjust later */
 
-   if((fptr = fopen(file, "r")) == NULL)
-      {
-	 fprintf(stderr, "LoadPrivateKey(): Couldn't open %s for reading\n", file);
-	 return -1;
-      }
+   if((fptr = fopen(file, "r")) == NULL) {
+      fprintf(stderr, "LoadPrivateKey(): Couldn't open %s for reading\n", file);
+      return -1;
+   }
 
    fgets(tempbuf, 20, fptr);
    if(strncmp(tempbuf, "RSA PRIVATE KEY", 15))
@@ -174,3 +173,14 @@ int LoadPart(FILE *fptr, void *dest, int size) {
 
    return 0;
 }
+
+char *MD5Hash(unsigned char *passphrase) {
+   MD5_CTX context;
+   unsigned char hash[16];
+
+   MD5Init(&context);
+   MD5Update(&context, passphrase, strlen((char *)passphrase));
+   MD5Final(hash, &context);
+
+   return base64_encode(hash, 16);
+}
