@@ -42,16 +42,14 @@ struct DHData *InitDH(R_RANDOM_STRUCT *randomStruct, int server, R_RSA_PRIVATE_K
   struct DHData *new;
   
   /* Set up the DH agreement and generate new public and private values. */
-  new = malloc(sizeof(struct DHData));
+  new = Malloc(sizeof(struct DHData));
   new->server = server;
-  if (R_SetupDHAgreement(new->publicValue, new->privateValue, DH_VAL_LEN, &PARAMS1, randomStruct) != 0)
-  {
+  if (R_SetupDHAgreement(new->publicValue, new->privateValue, DH_VAL_LEN, &PARAMS1, randomStruct) != 0) {
     free(new);
     return NULL;
   }
   /* If this is the server, sign the public value. */
-  if (server == 1)
-  {
+  if (server == 1) {
     MD5_CTX MD5Ctx;
     unsigned char digest[16];
     unsigned int siglen;
@@ -59,8 +57,7 @@ struct DHData *InitDH(R_RANDOM_STRUCT *randomStruct, int server, R_RSA_PRIVATE_K
     MD5Init(&MD5Ctx);
     MD5Update(&MD5Ctx, new->publicValue, DH_VAL_LEN);
     MD5Final(digest, &MD5Ctx);
-    if (RSAPrivateEncrypt(new->signature, &siglen, digest, 16, privateKey) != 0)
-    {
+    if (RSAPrivateEncrypt(new->signature, &siglen, digest, 16, privateKey) != 0) {
       free(new);
       return NULL;
     }
@@ -91,8 +88,7 @@ int ComputeDH(unsigned char *data, struct DHData *kexData, R_RSA_PUBLIC_KEY *pub
 
   memcpy(otherpubval, data, DH_VAL_LEN);
   /* If this is the client, check the server's signature. */
-  if (kexData->server == 0)
-  {
+  if (kexData->server == 0) {
     unsigned char digest[16], otherdigest[MAX_SIGNATURE_LEN];
     unsigned int otherdigestlen;
     MD5_CTX MD5Ctx;
