@@ -13,6 +13,10 @@
  #define isblank(c)	(((c) == ' ') || ((c) == '\t'))
 #endif
 
+#ifndef have_flock
+ #define flock(a, b)	0
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
@@ -20,8 +24,8 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <time.h>
-#include <syslog.h>
 #include <signal.h>
+#include <sys/file.h>
 #include <sys/param.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -37,6 +41,7 @@
 #include "global.h"
 #include "rsaref.h"
 #include "rsa.h"
+#include "common.h"
 #include "base64.h"
 #include "crypto.h"
 #include "arcfour.h"
@@ -45,10 +50,12 @@
 #include "netwrap.h"
 #include "error.h"
 #include "dh.h"
+#include "log.h"
+#include "conf.h"
 
 #include "compat.h"
 
-#define VERSION                 "0.4"
+#define VERSION                 "0.5"
 
 #ifndef MAX
  #define MAX(x, y)       ((x) > (y) ? (x) : (y))
@@ -61,9 +68,9 @@
  #define SHUT_RD 0
 #endif
 
-extern int h_errno;
 extern int debug;
 extern int daemon_proc;
+extern int do_log;
 extern char *seedfile;
 
 #endif
