@@ -49,7 +49,7 @@ int w_listen(char *host, const char *serv, int socktype) {
    if(host) {
       ipv6host(host);	/* Strip eventual epifix	*/
       if((hst = gethostbyname(host)) == NULL) {
-	 fprintf(stderr, "w_listen: error resolving %s: %d\n", host, h_errno);
+	 fprintf(stderr, "w_listen: error resolving %s.\n", host);
 	    exit(-1);
       }
       listenhost = *(int *)hst->h_addr_list[0];
@@ -81,7 +81,7 @@ int w_listen(char *host, const char *serv, int socktype) {
 
 char *w_sock_ntop(const struct sockaddr *sa) {
    char portstr[7];
-   static char str[40];
+   char str[40];
 
    switch(sa->sa_family) {
       case AF_INET: {
@@ -90,23 +90,40 @@ char *w_sock_ntop(const struct sockaddr *sa) {
 	 strcpy(str, inet_ntoa(sin->sin_addr));
 	 snprintf(portstr, sizeof(portstr), "#%d", ntohs(sin->sin_port));
 	 strcat(str, portstr);
-	 return(str);
+	 return strdup(str);
       }
 
 #ifdef IPV6
       case AF_INET6: {
 	 struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) sa;
 
-	 inet_ntop(AF_INET6, &sin6->sin6_addr, str, sizeof(str));
+	 if(IN6_IS_ADDR_V4MAPPED(&sin6->sin6_addr))
+	    inet_ntop(AF_INET, (struct in_addr *)(((char *)&sin6->sin6_addr)+12), str, sizeof(str));
+	 else
+	    inet_ntop(AF_INET6, &sin6->sin6_addr, str, sizeof(str));
 	 snprintf(portstr, sizeof(portstr), "#%d", ntohs(sin6->sin6_port));
 	 strcat(str, portstr);
-	 return(str);
+	 return strdup(str);
       }
 #endif
    }
    return NULL;	/* Never reached */
 }
 
+char *print_sock(int sd) {
+   int len = 100;
+   struct sockaddr *sv = Malloc(len);
+   char *ret;
+
+   if(getpeername(sd, sv, &len) != 0) {
+      free(sv);
+      return strdup("(null)");
+   }
+   ret = w_sock_ntop(sv);
+   free(sv);
+   return ret;
+}
+
 struct sockaddr *w_setsockaddr(void *destination, short port, socklen_t *destlen, int inet6) {
    struct sockaddr *dest;   
 
@@ -116,8 +133,7 @@ struct sockaddr *w_setsockaddr(void *destination, short port, socklen_t *destlen
    *destlen = sizeof(struct sockaddr_in);
 #endif
 
-   if((dest = calloc(1, *destlen)) == NULL)
-      err_quit("w_connect: Memory allocation error");
+   dest = Calloc(1, *destlen);
 
    if(inet6 == 0) {
       ((struct sockaddr_in *)dest)->sin_family = AF_INET;
