@@ -5,14 +5,11 @@
 void oob_add(int **posarray, int *size, int *used, int pos) {
    
    if(*size == *used) {
-      if((*posarray = realloc(*posarray, *size + 1)) == NULL)
-	 err_sys("add_oob: Memory allocation error");
-     
+      *posarray = Realloc(*posarray, *size + 1);
       *size += 1;
    }
 
-   if(debug)
-      err_msg("oob_add: Added OOB byte at position %d to %p[%d]", pos, *posarray, *used);
+   err_msg("oob_add: Added OOB byte at position %d to %p[%d]", pos, *posarray, *used);
 
    posarray[0][*used] = pos;
    *used += 1;
@@ -23,8 +20,7 @@ void oob_shift(int *posarray, int *used) {
 
     *used -= 1;
 
-    if(debug)
-       err_msg("oob_shift: Shifting OOB queue: New usedcount: %d", *used);
+    err_msg("oob_shift: Shifting OOB queue: New usedcount: %d", *used);
 
     memmove(posarray, posarray + 1, *used * sizeof(int));
 }
